/* eslint-env jest */
const { lint } = require('stylelint');
const path = require('path');
const chalk = require('chalk');

const root = path.resolve(__dirname, '../');

const config = {
  plugins: [path.resolve(root, 'src/plugins/styling-hooks-pattern.js')],
  rules: {
    'sds-stylelint-plugin/styling-hooks-pattern': [true],
  },
};

jest.mock('chalk', () => ({
  bold: jest.fn((text) => text),
  cyan: jest.fn((text) => text),
  redBright: jest.fn((text) => text),
}));

describe('sds-stylelint-config', () => {
  it('validate namespace', async () => {
    const {
      results: [{ warnings }],
    } = await lint({
      code: '.button { color: var(--sds-c-button-color-background); }',
      config,
    });

    expect(warnings).toHaveLength(0);
  });
  it('invalidate namespace', async () => {
    const {
      results: [{ warnings }],
    } = await lint({
      code: '.button { color: var(--aaa-c-button-color-background); }',
      config,
    });
    expect(warnings).toHaveLength(1);
    const [{ text }] = warnings;
    expect(text).toBe(
      '1 found on --aaa-c-button-color-background fails naming pattern. 1.) Expected ns value(s) of "sds,slds,dx,dxp,com,lume" but received "aaa". (sds-stylelint-plugin/styling-hooks-pattern)',
    );
  });
  it('invalidate scope', async () => {
    const {
      results: [{ warnings }],
    } = await lint({
      code: '.button { color: var(--sds-c-button-spacing-inline); }',
      config,
    });
    expect(warnings).toHaveLength(0);
  });
  xit('invalidate scope', async () => {
    const {
      results: [{ warnings }],
    } = await lint({
      code: '.button { color: var(--sds-m-button-spacing-inline); }',
      config,
    });
    expect(warnings).toHaveLength(1);
    const [{ text }] = warnings;
    expect(text).toBe(
      '1 found on --sds-m-button-spacing-inline fails naming pattern. 1.) Expected scope value(s) of "g,s,c" but received "m". (sds-stylelint-plugin/styling-hooks-pattern)',
    );
  });
  describe('component level styling hooks', () => {
    it('validate context', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-color-background); }',
        config,
      });
      expect(warnings).toHaveLength(0);
    });
    it('invalidate context', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-foo-color-background); }',
        config,
      });
      expect(warnings).toHaveLength(1);
      const [{ text }] = warnings;
      expect(text).toBe(
        `1 found on --sds-c-foo-color-background fails naming pattern. 1.) Expected context value(s) of "button" but received "foo". (sds-stylelint-plugin/styling-hooks-pattern)`,
      );
    });
    it('validate element', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-icon-color-foreground); }',
        config,
      });
      expect(warnings).toHaveLength(0);
    });
    it('validate category', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-color-foreground); }',
        config,
      });
      expect(warnings).toHaveLength(0);
    });
    it('invalidate category', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-dooooooo-foreground); }',
        config,
      });
      expect(warnings).toHaveLength(1);
      const [{ text }] = warnings;
      expect(text).toBe(
        '1 found on --sds-c-button-dooooooo-foreground fails naming pattern. 1.) Expected category value(s) of "color,font,radius,shadow,sizing,spacing,position" but received "dooooooo". (sds-stylelint-plugin/styling-hooks-pattern)',
      );
    });
    it('validate category with an element', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-icon-color-foreground); }',
        config,
      });
      expect(warnings).toHaveLength(0);
    });
    it('invalidate category with an element', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-icon-dooooooo); }',
        config,
      });
      expect(warnings).toHaveLength(1);
      const [{ text }] = warnings;
      expect(text).toBe(
        '1 found on --sds-c-button-icon-dooooooo fails naming pattern. 1.) [Analysis Warning]: We received "dooooooo" and have low confidence in determining the category. Possible values are color,font,radius,shadow,sizing,spacing,position. (sds-stylelint-plugin/styling-hooks-pattern)',
      );
    });
    it('invalidate category with a variant and element', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-brand-icon-dooooooo-foreground); }',
        config,
      });
      expect(warnings).toHaveLength(1);
      const [{ text }] = warnings;
      expect(text).toBe(
        '1 found on --sds-c-button-brand-icon-dooooooo-foreground fails naming pattern. 1.) Expected category value(s) of "color,font,radius,shadow,sizing,spacing,position" but received "dooooooo". (sds-stylelint-plugin/styling-hooks-pattern)',
      );
    });
    it('validate variant', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-brand-color-background); }',
        config,
      });
      expect(warnings).toHaveLength(0);
    });
    it('validate variant with element', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-brand-icon-color-foreground); }',
        config,
      });
      expect(warnings).toHaveLength(0);
    });
    it('validate property', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-color-background); }',
        config,
      });
      expect(warnings).toHaveLength(0);
    });
    it('invalidate property', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-color-lol); }',
        config,
      });
      expect(warnings).toHaveLength(1);
      const [{ text }] = warnings;
      expect(text).toBe(
        '1 found on --sds-c-button-color-lol fails naming pattern. 1.) Expected property value(s) of "background,foreground,border,style,size,family,weight,lineheight,decoration,align,width,height,zindex" but received "lol". (sds-stylelint-plugin/styling-hooks-pattern)',
      );
    });
    it('validate attribute', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-spacing-blockstart); }',
        config,
      });
      expect(warnings).toHaveLength(0);
    });
    it('invalidate attribute,state,or psuedoState (fuzzy results)', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-color-background-red); }',
        config,
      });
      expect(warnings).toHaveLength(1);
      const [{ text }] = warnings;
      expect(text).toBe(
        '1 found on --sds-c-button-color-background-red fails naming pattern. 1.) [Analysis Warning]: We received "red" and have low confidence in determining if this is an attribute, state, or psuedoState. Possible values are inline,inlinestart,inlineend,block,blockstart,blockend,start,startstart,startend,end,endstart,endend,gap,max,min,checked,pressed,disabled,readonly,valid,invalid,indeterminate,error,placeholder,selected,hover,focus,active,visited. (sds-stylelint-plugin/styling-hooks-pattern)',
      );
    });
    it('invalidate attribute with state', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-color-background-red-disabled); }',
        config,
      });
      expect(warnings).toHaveLength(1);
      const [{ text }] = warnings;
      expect(text).toBe(
        '1 found on --sds-c-button-color-background-red-disabled fails naming pattern. 1.) Expected attribute value(s) of "inline,inlinestart,inlineend,block,blockstart,blockend,start,startstart,startend,end,endstart,endend,gap,max,min" but received "red". (sds-stylelint-plugin/styling-hooks-pattern)',
      );
    });
    it('invalidate attribute with invalid state and invalid psuedoState', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-color-background-red-disabledddd-hoverrr); }',
        config,
      });
      expect(warnings).toHaveLength(1);
      const [{ text }] = warnings;
      expect(text).toBe(
        '3 found on --sds-c-button-color-background-red-disabledddd-hoverrr fails naming pattern. 1.) Expected attribute value(s) of "inline,inlinestart,inlineend,block,blockstart,blockend,start,startstart,startend,end,endstart,endend,gap,max,min" but received "red". 2.) Expected state value(s) of "checked,pressed,disabled,readonly,valid,invalid,indeterminate,error,placeholder,selected" but received "disabledddd". 3.) Expected psuedoState value(s) of "hover,focus,active,visited" but received "hoverrr". (sds-stylelint-plugin/styling-hooks-pattern)',
      );
    });
    it('validate state', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-color-background-disabled); }',
        config,
      });
      expect(warnings).toHaveLength(0);
    });
    it('invalidate state', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-color-background-disabledddd-hover); }',
        config,
      });
      expect(warnings).toHaveLength(1);
      const [{ text }] = warnings;
      expect(text).toBe(
        '1 found on --sds-c-button-color-background-disabledddd-hover fails naming pattern. 1.) Expected attribute value(s) of "inline,inlinestart,inlineend,block,blockstart,blockend,start,startstart,startend,end,endstart,endend,gap,max,min" but received "disabledddd". (sds-stylelint-plugin/styling-hooks-pattern)',
      );
    });
    it('fuzzy results: invalidate category, property', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-font1-style1); }',
        config,
      });
      expect(warnings).toHaveLength(1);
      const [{ text }] = warnings;
      expect(text).toBe(
        '1 found on --sds-c-button-font1-style1 fails naming pattern. 1.) [Analysis Warning]: We received "style1" and have low confidence in determining the category. Possible values are color,font,radius,shadow,sizing,spacing,position. (sds-stylelint-plugin/styling-hooks-pattern)',
      );
    });
    it('fuzzy results: invalidate property, attribute', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-font-style1-large1); }',
        config,
      });
      expect(warnings).toHaveLength(1);
      const [{ text }] = warnings;
      expect(text).toBe(
        '2 found on --sds-c-button-font-style1-large1 fails naming pattern. 1.) Expected property value(s) of "background,foreground,border,style,size,family,weight,lineheight,decoration,align,width,height,zindex" but received "style1". 2.) [Analysis Warning]: We received "large1" and have low confidence in determining if this is an attribute, state, or psuedoState. Possible values are inline,inlinestart,inlineend,block,blockstart,blockend,start,startstart,startend,end,endstart,endend,gap,max,min,checked,pressed,disabled,readonly,valid,invalid,indeterminate,error,placeholder,selected,hover,focus,active,visited. (sds-stylelint-plugin/styling-hooks-pattern)',
      );
    });
    it('fuzzy results: invalidate attribute, state', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-font-style-large1-hover1); }',
        config,
      });
      expect(warnings).toHaveLength(1);
      const [{ text }] = warnings;
      expect(text).toBe(
        '2 found on --sds-c-button-font-style-large1-hover1 fails naming pattern. 1.)  2.) [Analysis Warning]: We received "large1" and have medium confidence that this is an attribute but might also be a state, or psuedoState. Possible values are inline,inlinestart,inlineend,block,blockstart,blockend,start,startstart,startend,end,endstart,endend,gap,max,min,checked,pressed,disabled,readonly,valid,invalid,indeterminate,error,placeholder,selected,hover,focus,active,visited. We received "hover1" and have medium confidence that this could be a state or a psuedoState. Possible values are checked,pressed,disabled,readonly,valid,invalid,indeterminate,error,placeholder,selected,hover,focus,active,visited. (sds-stylelint-plugin/styling-hooks-pattern)',
      );
    });
    it('invalidate state, psuedoState', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-spacing-inline-disabled1-hover1); }',
        config,
      });
      expect(warnings).toHaveLength(1);
      const [{ text }] = warnings;
      expect(text).toBe(
        '2 found on --sds-c-button-spacing-inline-disabled1-hover1 fails naming pattern. 1.) Expected state value(s) of "checked,pressed,disabled,readonly,valid,invalid,indeterminate,error,placeholder,selected" but received "disabled1". 2.) Expected psuedoState value(s) of "hover,focus,active,visited" but received "hover1". (sds-stylelint-plugin/styling-hooks-pattern)',
      );
    });
    it('fuzzy results: invalidate category, attribute, state with element and variant', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--sds-c-button-foo-bar-font1-style1-placeholder1); }',
        config,
      });
      expect(warnings).toHaveLength(1);
      const [{ text }] = warnings;
      expect(text).toBe(
        '3 found on --sds-c-button-foo-bar-font1-style1-placeholder1 fails naming pattern. 1.) Expected category value(s) of "color,font,radius,shadow,sizing,spacing,position" but received "font1". 2.) Expected property value(s) of "background,foreground,border,style,size,family,weight,lineheight,decoration,align,width,height,zindex" but received "style1". 3.) [Analysis Warning]: We received "placeholder1" and have low confidence in determining if this is an attribute, state, or psuedoState. Possible values are inline,inlinestart,inlineend,block,blockstart,blockend,start,startstart,startend,end,endstart,endend,gap,max,min,checked,pressed,disabled,readonly,valid,invalid,indeterminate,error,placeholder,selected,hover,focus,active,visited. (sds-stylelint-plugin/styling-hooks-pattern)',
      );
    });
    it('ignores private custom properties', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--_foo); }',
        config,
      });

      expect(warnings).toHaveLength(0);
    });
    it('ignores private custom properties using custom private syntax', async () => {
      const {
        results: [{ warnings }],
      } = await lint({
        code: '.button { color: var(--_^foo); }',
        config: {
          plugins: [path.resolve(root, 'src/plugins/styling-hooks-pattern.js')],
          rules: {
            'sds-stylelint-plugin/styling-hooks-pattern': [true, { privateSyntax: '_^' }],
          },
        },
      });

      expect(warnings).toHaveLength(0);
    });
  });
});
