const path = require('path');
const stylelint = require('stylelint');
const fs = require('fs-extra');
const valueParser = require('postcss-value-parser');
const chalk = require('chalk');
const { isValidCustomProperty } = require('../utils');

const ruleName = 'sds-stylelint-plugin/styling-hooks-component-api';

const componentApi = (primary, options) => {
  return (root, result) => {
    const validOptions = stylelint.utils.validateOptions(result, ruleName, {
      actual: primary,
      possible: [true],
    });
    if (!validOptions) {
      return;
    }
    const context = root.source.input.file ? path.parse(root.source.input.file).name.split('.')[0] : null;
    const dir = root.source.input.file ? path.dirname(root.source.input.file) : null;
    if (fs.existsSync(`${dir}/${context}.metadata.json`)) {
      const metadata = fs.readJSONSync(`${dir}/${context}.metadata.json`, 'utf8');
      const missingHooks = new Set(metadata.css.hooks);

      root.walkDecls((decl) => {
        const parsedValue = valueParser(decl.value);
        const privateSyntax = (options && options.privateSyntax) || metadata.privateSyntax;

        parsedValue.walk(async (node) => {
          if (node.type !== 'word') return;

          if (
            isValidCustomProperty({ value: node.value, privateSyntax }) &&
            node.value.startsWith('--slds-c-')
          ) {
            // Check for deprecated values
            for (const deprecated of Object.keys(metadata.css.deprecated.hooks)) {
              if (node.value.includes(deprecated)) {
                stylelint.utils.report({
                  ruleName,
                  result,
                  message: `${chalk.black.yellowBright(
                    'Deprecation Warning',
                  )} The custom property "${chalk.cyan(
                    node.value,
                  )}" has been deprecated, please use "${chalk.greenBright(
                    metadata.css.deprecated.hooks[deprecated],
                  )}" instead.`,
                  node: decl,
                });
              }
            }

            for (const hook of metadata.css.hooks) {
              if (node.value.includes(hook)) {
                missingHooks.delete(hook);
                return;
              }
            }
          }
        });
      });

      root.walkRules((rule) => {
        const { selector } = rule;
        for (const deprecatedRule of Object.keys(metadata.css.deprecated.selectors)) {
          const sanitizedSelector = selector.replace(/\.|#/g, '');
          const sanitizedDeprecatedRule = deprecatedRule.replace(/\.|#/g, '');
          if (sanitizedSelector === sanitizedDeprecatedRule) {
            stylelint.utils.report({
              ruleName,
              result,
              message: `${chalk.black.yellowBright('Deprecation Warning')} The selector "${chalk.cyan(
                selector,
              )}" has been deprecated, please use "${chalk.greenBright(
                metadata.css.deprecated.selectors[deprecatedRule].replacement,
              )}" instead.`,
              node: rule,
            });
          }
        }
      });

      if (missingHooks.size > 0) {
        stylelint.utils.report({
          ruleName,
          result,
          message: `${chalk.yellowBright('API Warning')} "${chalk.cyan(
            Array.from(missingHooks).join(', '),
          )}" was not found in ${chalk.greenBright(
            context,
          )}.css and are flagged as public APIs. If you believe this to be an error, please reach out to the UX Foundations team.`,
          // node: null,
          line: 1,
          endLine: 1,
        });
      }
    }
  };
};

componentApi.ruleName = ruleName;

module.exports = stylelint.createPlugin(ruleName, componentApi);
